@@ -124,10 +124,10 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int6
                 Command_setEnv(C, "MONIT_DESCRIPTION", c == S->start ? "Started" : c == S->stop ? "Stopped" : "Restarted");
                 switch (S->type) {
                         case Service_Process:
-                                Command_vSetEnv(C, "MONIT_PROCESS_PID", "%d", S->inf->priv.process.pid);
-                                Command_vSetEnv(C, "MONIT_PROCESS_MEMORY", "%llu", (unsigned long long)((double)S->inf->priv.process.mem / 1024.));
-                                Command_vSetEnv(C, "MONIT_PROCESS_CHILDREN", "%d", S->inf->priv.process.children);
-                                Command_vSetEnv(C, "MONIT_PROCESS_CPU_PERCENT", "%.1f", S->inf->priv.process.cpu_percent);
+                                Command_vSetEnv(C, "MONIT_PROCESS_PID", "%d", S->inf.process->pid);
+                                Command_vSetEnv(C, "MONIT_PROCESS_MEMORY", "%llu", (unsigned long long)((double)S->inf.process->mem / 1024.));
+                                Command_vSetEnv(C, "MONIT_PROCESS_CHILDREN", "%d", S->inf.process->children);
+                                Command_vSetEnv(C, "MONIT_PROCESS_CPU_PERCENT", "%.1f", S->inf.process->cpu_percent);
                                 break;
                         case Service_Program:
                                 Command_vSetEnv(C, "MONIT_PROGRAM_STATUS", "%d", S->program->exitStatus);
@@ -200,7 +200,7 @@ static State_Type _check(Service_T s) {
         Monitor_Mode original = s->mode;
         s->mode = Monitor_Passive;
         rv = s->check(s);
-        if (s->type == Service_Program) {
+        if (s->type == Service_Program && s->program->P) {
                 // check program executes the program and needs to be called again to collect the exit value and evaluate the status
                 int64_t timeout = s->program->timeout * USEC_PER_MSEC;
                 do {
@@ -385,7 +385,6 @@ static boolean_t _doDepend(Service_T s, Action_Type action, boolean_t unmonitor)
         for (Service_T child = servicelist; child; child = child->next) {
                 for (Dependant_T d = child->dependantlist; d; d = d->next) {
                         if (IS(d->dependant, s->name)) {
-                                child->doaction = Action_Ignored;
                                 if (action == Action_Start) {
                                         // (re)start children only if it's monitoring is enabled (we keep monitoring flag during restart, allowing to restore original pre-restart configuration)
                                         if (child->monitor != Monitor_Not && ! _doStart(child))
@@ -407,6 +406,9 @@ static boolean_t _doDepend(Service_T s, Action_Type action, boolean_t unmonitor)
                                                 }
                                         }
                                 }
+                                if (child->doaction == action) {
+                                        child->doaction = Action_Ignored;
+                                }
                                 break;
                         }
                 }
@@ -456,7 +458,6 @@ boolean_t control_service(const char *S, Action_Type A) {
                 LogError("Service '%s' -- doesn't exist\n", S);
                 return false;
         }
-        s->doaction = Action_Ignored;
         switch (A) {
                 case Action_Start:
                         rv = _doStart(s);
@@ -500,7 +501,10 @@ boolean_t control_service(const char *S, Action_Type A) {
 
                 default:
                         LogError("Service '%s' -- invalid action %d\n", S, A);
-                        return false;
+                        rv = false;
+        }
+        if (s->doaction == A) {
+                s->doaction = Action_Ignored;
         }
         return rv;
 }
