@@ -82,9 +82,7 @@
  * @see https://bitbucket.org/tildeslash/monit/commits/cd545838378517f84bdb0989cadf461a19d8ba11
  */
 void init_env() {
-        // Close all descriptors except stdio
-        for (int i = 3, descriptors = getdtablesize(); i < descriptors; i++)
-                close(i);
+        Util_closeFds();
         // Ensure that std descriptors (0, 1 and 2) are open
         for (int i = 0; i < 3; i++) {
                 struct stat st;
