@@ -85,14 +85,14 @@ EventTable_T Event_Table[] = {
         {Event_Content,    "Content failed",            "Content succeeded",          "Content match",            "Content doesn't match"},
         {Event_Data,       "Data access error",         "Data access succeeded",      "Data access changed",      "Data access not changed"},
         {Event_Exec,       "Execution failed",          "Execution succeeded",        "Execution changed",        "Execution not changed"},
-        {Event_Fsflag,     "Filesystem flags failed",   "Filesystem flags succeeded", "Filesystem flags changed", "Filesystem flags not changed"},
+        {Event_FsFlag,     "Filesystem flags failed",   "Filesystem flags succeeded", "Filesystem flags changed", "Filesystem flags not changed"},
         {Event_Gid,        "GID failed",                "GID succeeded",              "GID changed",              "GID not changed"},
         {Event_Heartbeat,  "Heartbeat failed",          "Heartbeat succeeded",        "Heartbeat changed",        "Heartbeat not changed"},
         {Event_Icmp,       "ICMP failed",               "ICMP succeeded",             "ICMP changed",             "ICMP not changed"},
         {Event_Instance,   "Monit instance failed",     "Monit instance succeeded",   "Monit instance changed",   "Monit instance not changed"},
         {Event_Invalid,    "Invalid type",              "Type succeeded",             "Type changed",             "Type not changed"},
         {Event_Link,       "Link down",                 "Link up",                    "Link changed",             "Link not changed"},
-        {Event_Nonexist,   "Does not exist",            "Exists",                     "Existence changed",        "Existence not changed"},
+        {Event_NonExist,   "Does not exist",            "Exists",                     "Existence changed",        "Existence not changed"},
         {Event_PacketIn,   "Download packets exceeded", "Download packets ok",        "Download packets changed", "Download packets not changed"},
         {Event_PacketOut,  "Upload packets exceeded",   "Upload packets ok",          "Upload packets changed",   "Upload packets not changed"},
         {Event_Permission, "Permission failed",         "Permission succeeded",       "Permission changed",       "Permission not changed"},
@@ -107,6 +107,7 @@ EventTable_T Event_Table[] = {
         {Event_Timestamp,  "Timestamp failed",          "Timestamp succeeded",        "Timestamp changed",        "Timestamp not changed"},
         {Event_Uid,        "UID failed",                "UID succeeded",              "UID changed",              "UID not changed"},
         {Event_Uptime,     "Uptime failed",             "Uptime succeeded",           "Uptime changed",           "Uptime not changed"},
+        {Event_Exist,      "Does exist",                "Exists not",                 "Existence changed",        "Existence not changed"},
         /* Virtual events */
         {Event_Null,       "No Event",                  "No Event",                   "No Event",                 "No Event"}
 };
@@ -329,18 +330,26 @@ static void _handleEvent(Service_T S, Event_T E) {
         }
 
         if (E->message) {
-                /* In the case that the service state is initializing yet and error
-                 * occurred, log it and exit. Succeeded events in init state are not
-                 * logged. Instance and action events are logged always with priority
-                 * info. */
-                if (E->state != State_Init || E->state_map & 0x1) {
-                        if (E->state == State_Succeeded || E->state == State_ChangedNot || E->id == Event_Instance || E->id == Event_Action)
+                if (E->id == Event_Instance || E->id == Event_Action) {
+                        // Instance and action events are logged always with priority info
+                        LogInfo("'%s' %s\n", S->name, E->message);
+                } else if (E->state == State_Succeeded || E->state == State_ChangedNot) {
+                        if (E->state_map & 0x1) {
+                                // Failure, but didn't reach the error threshold yet
+                                LogWarning("'%s' %s\n", S->name, E->message);
+                        } else {
+                                // Success
                                 LogInfo("'%s' %s\n", S->name, E->message);
-                        else
-                                LogError("'%s' %s\n", S->name, E->message);
-                }
-                if (E->state == State_Init)
+                        }
+                } else if (E->state == State_Init) {
+                        if (E->state_map & 0x1) {
+                                // Log error which occur while the service is initializing as warnings, success is not logged in the initializing state
+                                LogWarning("'%s' %s\n", S->name, E->message);
+                        }
                         return;
+                } else {
+                        LogError("'%s' %s\n", S->name, E->message);
+                }
         }
 
         if (E->state == State_Failed || E->state == State_Changed) {
@@ -498,7 +507,7 @@ Action_Type Event_get_action(Event_T E) {
 
 /**
  * Get a textual description of actual event action. For instance if the
- * event type is possitive Event_Nonexist, the textual description of
+ * event type is possitive Event_NonExist, the textual description of
  * failed state related action is "restart". Likewise if the event type is
  * negative Event_Checksumthe textual description of recovery related action
  * is "alert" and so on.
