@@ -31,19 +31,19 @@
 typedef enum {
         Event_Null       = 0x0,
         Event_Checksum   = 0x1,
-        Event_Resource   = 0x2,
+        Event_Resource   = 0x2, //FIXME: split to more specific events (cpu, totalcpu, mem, totalmem, loadaverage, space, inode, ...)
         Event_Timeout    = 0x4,
-        Event_Timestamp  = 0x8,
+        Event_Timestamp  = 0x8, //FIXME: split to more specific events (atime, mtime, ctime)
         Event_Size       = 0x10,
         Event_Connection = 0x20,
         Event_Permission = 0x40,
         Event_Uid        = 0x80,
         Event_Gid        = 0x100,
-        Event_Nonexist   = 0x200,
+        Event_NonExist   = 0x200,
         Event_Invalid    = 0x400,
         Event_Data       = 0x800,
         Event_Exec       = 0x1000,
-        Event_Fsflag     = 0x2000,
+        Event_FsFlag     = 0x2000,
         Event_Icmp       = 0x4000,
         Event_Content    = 0x8000,
         Event_Instance   = 0x10000,
@@ -53,13 +53,14 @@ typedef enum {
         Event_Heartbeat  = 0x100000,
         Event_Status     = 0x200000,
         Event_Uptime     = 0x400000,
-        Event_Link       = 0x800000,
+        Event_Link       = 0x800000, //FIXME: split to more specific events (link status, link errors)
         Event_Speed      = 0x1000000,
         Event_Saturation = 0x2000000,
         Event_ByteIn     = 0x4000000,
         Event_ByteOut    = 0x8000000,
         Event_PacketIn   = 0x10000000,
         Event_PacketOut  = 0x20000000,
+        Event_Exist      = 0x40000000,
         Event_All        = 0x7FFFFFFF
 } Event_Type;
 
@@ -124,7 +125,7 @@ Action_Type Event_get_action(Event_T E);
 
 /**
  * Get a textual description of actual event action. For instance if the
- * event type is possitive Event_Nonexist, the textual description of
+ * event type is possitive Event_NonExist, the textual description of
  * failed state related action is "restart". Likewise if the event type is
  * negative Event_Checksum the textual description of recovery related action
  * is "alert" and so on.
