@@ -117,7 +117,7 @@ static void document_head(StringBuffer_T B, int V, const char *myip) {
 
         if (Run.httpd.flags & Httpd_Net || Run.httpd.flags & Httpd_Unix) {
                 if (Run.httpd.flags & Httpd_Net)
-                        StringBuffer_append(B, "<httpd><address>%s</address><port>%d</port><ssl>%d</ssl></httpd>", Run.httpd.socket.net.address ? Run.httpd.socket.net.address : myip ? myip : "", Run.httpd.socket.net.port, Run.httpd.flags & Httpd_Ssl);
+                        StringBuffer_append(B, "<httpd><address>%s</address><port>%d</port><ssl>%d</ssl></httpd>", Run.httpd.socket.net.address ? Run.httpd.socket.net.address : myip ? myip : "", Run.httpd.socket.net.port, Run.httpd.socket.net.ssl.flags & SSL_Enabled);
                 else if (Run.httpd.flags & Httpd_Unix)
                         StringBuffer_append(B, "<httpd><unixsocket>%s</unixsocket></httpd>", Run.httpd.socket.unix.path ? Run.httpd.socket.unix.path : "");
 
@@ -140,9 +140,9 @@ static void document_head(StringBuffer_T B, int V, const char *myip) {
                             systeminfo.uname.release,
                             systeminfo.uname.version,
                             systeminfo.uname.machine,
-                            systeminfo.cpus,
-                            (unsigned long long)((double)systeminfo.mem_max / 1024.),   // Send as kB for backward compatibility
-                            (unsigned long long)((double)systeminfo.swap_max / 1024.)); // Send as kB for backward compatibility
+                            systeminfo.cpu.count,
+                            (unsigned long long)((double)systeminfo.memory.size / 1024.),   // Send as kB for backward compatibility
+                            (unsigned long long)((double)systeminfo.swap.size / 1024.)); // Send as kB for backward compatibility
 }
 
 
@@ -155,6 +155,30 @@ static void document_foot(StringBuffer_T B) {
 }
 
 
+static void _ioStatistics(StringBuffer_T B, const char *name, IOStatistics_T statistics) {
+        StringBuffer_append(B, "<%s>", name);
+        if (Statistics_initialized(&(statistics->bytes))) {
+                StringBuffer_append(B,
+                        "<bytes>"
+                        "<count>%.0lf</count>"     // bytes per second
+                        "<total>%"PRIu64"</total>" // bytes since boot
+                        "</bytes>",
+                        Statistics_deltaNormalize(&(statistics->bytes)),
+                        Statistics_raw(&(statistics->bytes)));
+        }
+        if (Statistics_initialized(&(statistics->operations))) {
+                StringBuffer_append(B,
+                        "<operations>"
+                        "<count>%.0lf</count>"     // operations per second
+                        "<total>%"PRIu64"</total>" // operations since boot
+                        "</operations>",
+                        Statistics_deltaNormalize(&(statistics->operations)),
+                        Statistics_raw(&(statistics->operations)));
+        }
+        StringBuffer_append(B, "</%s>", name);
+}
+
+
 /**
  * Prints a service status into the given buffer.
  * @param S Service object
@@ -198,15 +222,21 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                         "<mode>%o</mode>"
                                         "<uid>%d</uid>"
                                         "<gid>%d</gid>"
-                                        "<timestamp>%lld</timestamp>"
+                                        "<timestamps>"
+                                        "<access>%"PRIu64"</access>"
+                                        "<change>%"PRIu64"</change>"
+                                        "<modify>%"PRIu64"</modify>"
+                                        "</timestamps>"
                                         "<size>%llu</size>",
-                                        S->inf->priv.file.mode & 07777,
-                                        (int)S->inf->priv.file.uid,
-                                        (int)S->inf->priv.file.gid,
-                                        (long long)S->inf->priv.file.timestamp,
-                                        (unsigned long long)S->inf->priv.file.size);
+                                        S->inf.file->mode & 07777,
+                                        (int)S->inf.file->uid,
+                                        (int)S->inf.file->gid,
+                                        S->inf.file->timestamp.access,
+                                        S->inf.file->timestamp.change,
+                                        S->inf.file->timestamp.modify,
+                                        (unsigned long long)S->inf.file->size);
                                 if (S->checksum)
-                                        StringBuffer_append(B, "<checksum type=\"%s\">%s</checksum>", checksumnames[S->checksum->type], S->inf->priv.file.cs_sum);
+                                        StringBuffer_append(B, "<checksum type=\"%s\">%s</checksum>", checksumnames[S->checksum->type], S->inf.file->cs_sum);
                                 break;
 
                         case Service_Directory:
@@ -214,11 +244,17 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                         "<mode>%o</mode>"
                                         "<uid>%d</uid>"
                                         "<gid>%d</gid>"
-                                        "<timestamp>%lld</timestamp>",
-                                        S->inf->priv.directory.mode & 07777,
-                                        (int)S->inf->priv.directory.uid,
-                                        (int)S->inf->priv.directory.gid,
-                                        (long long)S->inf->priv.directory.timestamp);
+                                        "<timestamps>"
+                                        "<access>%"PRIu64"</access>"
+                                        "<change>%"PRIu64"</change>"
+                                        "<modify>%"PRIu64"</modify>"
+                                        "</timestamps>",
+                                        S->inf.directory->mode & 07777,
+                                        (int)S->inf.directory->uid,
+                                        (int)S->inf.directory->gid,
+                                        S->inf.directory->timestamp.access,
+                                        S->inf.directory->timestamp.change,
+                                        S->inf.directory->timestamp.modify);
                                 break;
 
                         case Service_Fifo:
@@ -226,41 +262,67 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                         "<mode>%o</mode>"
                                         "<uid>%d</uid>"
                                         "<gid>%d</gid>"
-                                        "<timestamp>%lld</timestamp>",
-                                        S->inf->priv.fifo.mode & 07777,
-                                        (int)S->inf->priv.fifo.uid,
-                                        (int)S->inf->priv.fifo.gid,
-                                        (long long)S->inf->priv.fifo.timestamp);
+                                        "<timestamps>"
+                                        "<access>%"PRIu64"</access>"
+                                        "<change>%"PRIu64"</change>"
+                                        "<modify>%"PRIu64"</modify>"
+                                        "</timestamps>",
+                                        S->inf.fifo->mode & 07777,
+                                        (int)S->inf.fifo->uid,
+                                        (int)S->inf.fifo->gid,
+                                        S->inf.fifo->timestamp.access,
+                                        S->inf.fifo->timestamp.change,
+                                        S->inf.fifo->timestamp.modify);
                                 break;
 
                         case Service_Filesystem:
                                 StringBuffer_append(B,
+                                        "<fstype>%s</fstype>"
+                                        "<fsflags>%s</fsflags>"
                                         "<mode>%o</mode>"
                                         "<uid>%d</uid>"
                                         "<gid>%d</gid>"
-                                        "<flags>%d</flags>"
                                         "<block>"
                                         "<percent>%.1f</percent>"
-                                        "<usage>%.1f</usage>"
-                                        "<total>%.1f</total>"
+                                        "<usage>%.1lf</usage>"
+                                        "<total>%.1lf</total>"
                                         "</block>",
-                                        S->inf->priv.filesystem.mode & 07777,
-                                        (int)S->inf->priv.filesystem.uid,
-                                        (int)S->inf->priv.filesystem.gid,
-                                        S->inf->priv.filesystem.flags,
-                                        S->inf->priv.filesystem.space_percent,
-                                        S->inf->priv.filesystem.f_bsize > 0 ? (double)S->inf->priv.filesystem.space_total / 1048576. * (double)S->inf->priv.filesystem.f_bsize : 0.,
-                                        S->inf->priv.filesystem.f_bsize > 0 ? (double)S->inf->priv.filesystem.f_blocks / 1048576. * (double)S->inf->priv.filesystem.f_bsize : 0.);
-                                if (S->inf->priv.filesystem.f_files > 0) {
+                                        S->inf.filesystem->object.type,
+                                        S->inf.filesystem->flags,
+                                        S->inf.filesystem->mode & 07777,
+                                        (int)S->inf.filesystem->uid,
+                                        (int)S->inf.filesystem->gid,
+                                        S->inf.filesystem->space_percent,
+                                        S->inf.filesystem->f_bsize > 0 ? (double)S->inf.filesystem->space_total / 1048576. * (double)S->inf.filesystem->f_bsize : 0.,
+                                        S->inf.filesystem->f_bsize > 0 ? (double)S->inf.filesystem->f_blocks / 1048576. * (double)S->inf.filesystem->f_bsize : 0.);
+                                if (S->inf.filesystem->f_files > 0) {
                                         StringBuffer_append(B,
                                                 "<inode>"
                                                 "<percent>%.1f</percent>"
                                                 "<usage>%lld</usage>"
                                                 "<total>%lld</total>"
                                                 "</inode>",
-                                                S->inf->priv.filesystem.inode_percent,
-                                                S->inf->priv.filesystem.inode_total,
-                                                S->inf->priv.filesystem.f_files);
+                                                S->inf.filesystem->inode_percent,
+                                                S->inf.filesystem->inode_total,
+                                                S->inf.filesystem->f_files);
+                                }
+                                _ioStatistics(B, "read", &(S->inf.filesystem->read));
+                                _ioStatistics(B, "write", &(S->inf.filesystem->write));
+                                boolean_t hasReadTime = Statistics_initialized(&(S->inf.filesystem->time.read));
+                                boolean_t hasWriteTime = Statistics_initialized(&(S->inf.filesystem->time.write));
+                                boolean_t hasWaitTime = Statistics_initialized(&(S->inf.filesystem->time.wait));
+                                boolean_t hasRunTime = Statistics_initialized(&(S->inf.filesystem->time.run));
+                                if (hasReadTime || hasWriteTime || hasWaitTime || hasRunTime) {
+                                        StringBuffer_append(B, "<servicetime>");
+                                        if (hasReadTime)
+                                                StringBuffer_append(B, "<read>%.3f</read>", Statistics_deltaNormalize(&(S->inf.filesystem->time.read)));
+                                        if (hasWriteTime)
+                                                StringBuffer_append(B, "<write>%.3f</write>", Statistics_deltaNormalize(&(S->inf.filesystem->time.write)));
+                                        if (hasWaitTime)
+                                                StringBuffer_append(B, "<wait>%.3f</wait>", Statistics_deltaNormalize(&(S->inf.filesystem->time.wait)));
+                                        if (hasRunTime)
+                                                StringBuffer_append(B, "<run>%.3f</run>", Statistics_deltaNormalize(&(S->inf.filesystem->time.run)));
+                                        StringBuffer_append(B, "</servicetime>");
                                 }
                                 break;
 
@@ -299,21 +361,21 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                         "</errors>"
                                         "</upload>"
                                         "</link>",
-                                        Link_getState(S->inf->priv.net.stats),
-                                        Link_getSpeed(S->inf->priv.net.stats),
-                                        Link_getDuplex(S->inf->priv.net.stats),
-                                        Link_getPacketsInPerSecond(S->inf->priv.net.stats),
-                                        Link_getPacketsInTotal(S->inf->priv.net.stats),
-                                        Link_getBytesInPerSecond(S->inf->priv.net.stats),
-                                        Link_getBytesInTotal(S->inf->priv.net.stats),
-                                        Link_getErrorsInPerSecond(S->inf->priv.net.stats),
-                                        Link_getErrorsInTotal(S->inf->priv.net.stats),
-                                        Link_getPacketsOutPerSecond(S->inf->priv.net.stats),
-                                        Link_getPacketsOutTotal(S->inf->priv.net.stats),
-                                        Link_getBytesOutPerSecond(S->inf->priv.net.stats),
-                                        Link_getBytesOutTotal(S->inf->priv.net.stats),
-                                        Link_getErrorsOutPerSecond(S->inf->priv.net.stats),
-                                        Link_getErrorsOutTotal(S->inf->priv.net.stats));
+                                        Link_getState(S->inf.net->stats),
+                                        Link_getSpeed(S->inf.net->stats),
+                                        Link_getDuplex(S->inf.net->stats),
+                                        Link_getPacketsInPerSecond(S->inf.net->stats),
+                                        Link_getPacketsInTotal(S->inf.net->stats),
+                                        Link_getBytesInPerSecond(S->inf.net->stats),
+                                        Link_getBytesInTotal(S->inf.net->stats),
+                                        Link_getErrorsInPerSecond(S->inf.net->stats),
+                                        Link_getErrorsInTotal(S->inf.net->stats),
+                                        Link_getPacketsOutPerSecond(S->inf.net->stats),
+                                        Link_getPacketsOutTotal(S->inf.net->stats),
+                                        Link_getBytesOutPerSecond(S->inf.net->stats),
+                                        Link_getBytesOutTotal(S->inf.net->stats),
+                                        Link_getErrorsOutPerSecond(S->inf.net->stats),
+                                        Link_getErrorsOutTotal(S->inf.net->stats));
                                 break;
 
                         case Service_Process:
@@ -324,12 +386,12 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                         "<euid>%d</euid>"
                                         "<gid>%d</gid>"
                                         "<uptime>%lld</uptime>",
-                                        S->inf->priv.process.pid,
-                                        S->inf->priv.process.ppid,
-                                        S->inf->priv.process.uid,
-                                        S->inf->priv.process.euid,
-                                        S->inf->priv.process.gid,
-                                        (long long)S->inf->priv.process.uptime);
+                                        S->inf.process->pid,
+                                        S->inf.process->ppid,
+                                        S->inf.process->uid,
+                                        S->inf.process->euid,
+                                        S->inf.process->gid,
+                                        (long long)S->inf.process->uptime);
                                 if (Run.flags & Run_ProcessEngineEnabled) {
                                         StringBuffer_append(B,
                                                 "<threads>%d</threads>"
@@ -344,15 +406,17 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                                 "<percent>%.1f</percent>"
                                                 "<percenttotal>%.1f</percenttotal>"
                                                 "</cpu>",
-                                                S->inf->priv.process.threads,
-                                                S->inf->priv.process.children,
-                                                S->inf->priv.process.mem_percent,
-                                                S->inf->priv.process.total_mem_percent,
-                                                (unsigned long long)((double)S->inf->priv.process.mem / 1024.),       // Send as kB for backward compatibility
-                                                (unsigned long long)((double)S->inf->priv.process.total_mem / 1024.), // Send as kB for backward compatibility
-                                                S->inf->priv.process.cpu_percent,
-                                                S->inf->priv.process.total_cpu_percent);
+                                                S->inf.process->threads,
+                                                S->inf.process->children,
+                                                S->inf.process->mem_percent,
+                                                S->inf.process->total_mem_percent,
+                                                (unsigned long long)((double)S->inf.process->mem / 1024.),       // Send as kB for backward compatibility
+                                                (unsigned long long)((double)S->inf.process->total_mem / 1024.), // Send as kB for backward compatibility
+                                                S->inf.process->cpu_percent,
+                                                S->inf.process->total_cpu_percent);
                                 }
+                                _ioStatistics(B, "read", &(S->inf.process->read));
+                                _ioStatistics(B, "write", &(S->inf.process->write));
                                 break;
 
                         default:
@@ -375,14 +439,21 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                             "<request><![CDATA[%s]]></request>"
                                             "<protocol>%s</protocol>"
                                             "<type>%s</type>"
-                                            "<responsetime>%.6f</responsetime>"
-                                            "</port>",
+                                            "<responsetime>%.6f</responsetime>",
                                             p->hostname ? p->hostname : "",
                                             p->target.net.port,
                                             Util_portRequestDescription(p),
                                             p->protocol->name ? p->protocol->name : "",
                                             Util_portTypeDescription(p),
                                             p->is_available == Connection_Ok ? p->response / 1000. : -1.); // We send the response time in [s] for backward compatibility (with microseconds precision)
+                        if (p->target.net.ssl.options.flags)
+                                StringBuffer_append(B,
+                                            "<certificate>"
+                                            "<valid>%d</valid>"
+                                            "</certificate>",
+                                            p->target.net.ssl.certificate.validDays);
+                        StringBuffer_append(B,
+                                            "</port>");
                 }
                 for (Port_T p = S->socketlist; p; p = p->next) {
                         StringBuffer_append(B,
@@ -422,15 +493,15 @@ static void status_service(Service_T S, StringBuffer_T B, int V) {
                                             systeminfo.loadavg[0],
                                             systeminfo.loadavg[1],
                                             systeminfo.loadavg[2],
-                                            systeminfo.total_cpu_user_percent > 0. ? systeminfo.total_cpu_user_percent : 0.,
-                                            systeminfo.total_cpu_syst_percent > 0. ? systeminfo.total_cpu_syst_percent : 0.,
+                                            systeminfo.cpu.usage.user > 0. ? systeminfo.cpu.usage.user : 0.,
+                                            systeminfo.cpu.usage.system > 0. ? systeminfo.cpu.usage.system : 0.,
 #ifdef HAVE_CPU_WAIT
-                                            systeminfo.total_cpu_wait_percent > 0. ? systeminfo.total_cpu_wait_percent : 0.,
+                                            systeminfo.cpu.usage.wait > 0. ? systeminfo.cpu.usage.wait : 0.,
 #endif
-                                            systeminfo.total_mem_percent,
-                                            (unsigned long long)((double)systeminfo.total_mem / 1024.),               // Send as kB for backward compatibility
-                                            systeminfo.total_swap_percent,
-                                            (unsigned long long)((double)systeminfo.total_swap / 1024.));             // Send as kB for backward compatibility
+                                            systeminfo.memory.usage.percent,
+                                            (unsigned long long)((double)systeminfo.memory.usage.bytes / 1024.),               // Send as kB for backward compatibility
+                                            systeminfo.swap.usage.percent,
+                                            (unsigned long long)((double)systeminfo.swap.usage.bytes / 1024.));             // Send as kB for backward compatibility
                 }
                 if (S->type == Service_Program && S->program->started) {
                         StringBuffer_append(B,
