@@ -99,15 +99,15 @@ static long cpu_syst_old = 0;
 
 boolean_t init_process_info_sysdep(void) {
         int mib[2] = {CTL_HW, HW_NCPU};
-        size_t len = sizeof(systeminfo.cpus);
-        if (sysctl(mib, 2, &systeminfo.cpus, &len, NULL, 0) == -1) {
+        size_t len = sizeof(systeminfo.cpu.count);
+        if (sysctl(mib, 2, &systeminfo.cpu.count, &len, NULL, 0) == -1) {
                 DEBUG("system statistic error -- cannot get cpu count: %s\n", STRERROR);
                 return false;
         }
 
         mib[1] = HW_PHYSMEM;
-        len    = sizeof(systeminfo.mem_max);
-        if (sysctl(mib, 2, &systeminfo.mem_max, &len, NULL, 0) == -1) {
+        len    = sizeof(systeminfo.memory.size);
+        if (sysctl(mib, 2, &systeminfo.memory.size, &len, NULL, 0) == -1) {
                 DEBUG("system statistic error -- cannot get real memory amount: %s\n", STRERROR);
                 return false;
         }
@@ -159,16 +159,18 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
         if (pflags & ProcessEngine_CollectCommandLine)
                 cmdline = StringBuffer_create(64);
         for (int i = 0; i < treesize; i++) {
-                pt[i].pid          = pinfo[i].kp_pid;
-                pt[i].ppid         = pinfo[i].kp_ppid;
-                pt[i].cred.uid     = pinfo[i].kp_ruid;
-                pt[i].cred.euid    = pinfo[i].kp_uid;
-                pt[i].cred.gid     = pinfo[i].kp_rgid;
-                pt[i].threads      = pinfo[i].kp_nthreads;
-                pt[i].uptime       = systeminfo.time / 10. - pinfo[i].kp_start.tv_sec;
-                pt[i].cpu.time     = (double)((pinfo[i].kp_lwp.kl_uticks + pinfo[i].kp_lwp.kl_sticks + pinfo[i].kp_lwp.kl_iticks) / 1000000.);
-                pt[i].memory.usage = (uint64_t)pinfo[i].kp_vm_rssize * (uint64_t)pagesize;
-                pt[i].zombie       = pinfo[i].kp_stat == SZOMB ? true : false;
+                pt[i].pid              = pinfo[i].kp_pid;
+                pt[i].ppid             = pinfo[i].kp_ppid;
+                pt[i].cred.uid         = pinfo[i].kp_ruid;
+                pt[i].cred.euid        = pinfo[i].kp_uid;
+                pt[i].cred.gid         = pinfo[i].kp_rgid;
+                pt[i].threads          = pinfo[i].kp_nthreads;
+                pt[i].uptime           = systeminfo.time / 10. - pinfo[i].kp_start.tv_sec;
+                pt[i].cpu.time         = (double)((pinfo[i].kp_lwp.kl_uticks + pinfo[i].kp_lwp.kl_sticks + pinfo[i].kp_lwp.kl_iticks) / 1000000.);
+                pt[i].memory.usage     = (uint64_t)pinfo[i].kp_vm_rssize * (uint64_t)pagesize;
+                pt[i].read.operations  = pinfo[i].kp_ru.ru_inblock;
+                pt[i].write.operations = pinfo[i].kp_ru.ru_oublock;
+                pt[i].zombie           = pinfo[i].kp_stat == SZOMB ? true : false;
                 if (pflags & ProcessEngine_CollectCommandLine) {
                         char **args = kvm_getargv(kvm_handle, &pinfo[i], 0);
                         if (args) {
@@ -178,7 +180,7 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
                                 if (StringBuffer_length(cmdline))
                                         pt[i].cmdline = Str_dup(StringBuffer_toString(StringBuffer_trim(cmdline)));
                         }
-                        if (! pt[i].cmdline || ! *pt[i].cmdline) {
+                        if (STR_UNDEF(pt[i].cmdline)) {
                                 FREE(pt[i].cmdline);
                                 pt[i].cmdline = Str_dup(pinfo[i].kp_comm);
                         }
@@ -231,29 +233,29 @@ boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
                 LogError("system statistic error -- wired memory usage statics error\n");
                 return false;
         }
-        si->total_mem = (uint64_t)(active + wired) * (uint64_t)pagesize;
+        si->memory.usage.bytes = (uint64_t)(active + wired) * (uint64_t)pagesize;
 
         /* Swap */
         unsigned int used;
         if (sysctlbyname("vm.swap_anon_use", &used, &len, NULL, 0) == -1) {
                 LogError("system statistic error -- cannot get swap usage: %s\n", STRERROR);
-                si->swap_max = 0;
+                si->swap.size = 0;
                 return false;
         }
-        si->total_swap = (uint64_t)used * (uint64_t)pagesize;
+        si->swap.usage.bytes = (uint64_t)used * (uint64_t)pagesize;
         if (sysctlbyname("vm.swap_cache_use", &used, &len, NULL, 0) == -1) {
                 LogError("system statistic error -- cannot get swap usage: %s\n", STRERROR);
-                si->swap_max = 0;
+                si->swap.size = 0;
                 return false;
         }
-        si->total_swap += (uint64_t)used * (uint64_t)pagesize;
+        si->swap.usage.bytes += (uint64_t)used * (uint64_t)pagesize;
         unsigned int free;
         if (sysctlbyname("vm.swap_size", &free, &len, NULL, 0) == -1) {
                 LogError("system statistic error -- cannot get swap usage: %s\n", STRERROR);
-                si->swap_max = 0;
+                si->swap.size = 0;
                 return false;
         }
-        si->swap_max = (uint64_t)free * (uint64_t)pagesize + si->total_swap;
+        si->swap.size = (uint64_t)free * (uint64_t)pagesize + si->swap.usage.bytes;
         return true;
 }
 
@@ -287,9 +289,9 @@ boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
         total     = total_new - total_old;
         total_old = total_new;
 
-        si->total_cpu_user_percent = (total > 0) ? (100. * (double)(cp_time[CP_USER] - cpu_user_old) / total) : -1.;
-        si->total_cpu_syst_percent = (total > 0) ? (100. * (double)(cp_time[CP_SYS] - cpu_syst_old) / total) : -1.;
-        si->total_cpu_wait_percent = 0.; /* there is no wait statistic available */
+        si->cpu.usage.user = (total > 0) ? (100. * (double)(cp_time[CP_USER] - cpu_user_old) / total) : -1.;
+        si->cpu.usage.system = (total > 0) ? (100. * (double)(cp_time[CP_SYS] - cpu_syst_old) / total) : -1.;
+        si->cpu.usage.wait = 0.; /* there is no wait statistic available */
 
         cpu_user_old = cp_time[CP_USER];
         cpu_syst_old = cp_time[CP_SYS];
