@@ -88,15 +88,15 @@ static unsigned maxslp;
 
 boolean_t init_process_info_sysdep(void) {
         int mib[2] = {CTL_HW, HW_NCPU};
-        size_t len = sizeof(systeminfo.cpus);
-        if (sysctl(mib, 2, &systeminfo.cpus, &len, NULL, 0) == -1) {
+        size_t len = sizeof(systeminfo.cpu.count);
+        if (sysctl(mib, 2, &systeminfo.cpu.count, &len, NULL, 0) == -1) {
                 DEBUG("system statistic error -- cannot get cpu count: %s\n", STRERROR);
                 return false;
         }
 
         mib[1] = HW_PHYSMEM;
-        len    = sizeof(systeminfo.mem_max);
-        if (sysctl(mib, 2, &systeminfo.mem_max, &len, NULL, 0) == -1) {
+        len    = sizeof(systeminfo.memory.size);
+        if (sysctl(mib, 2, &systeminfo.memory.size, &len, NULL, 0) == -1) {
                 DEBUG("system statistic error -- cannot get real memory amount: %s\n", STRERROR);
                 return false;
         }
@@ -169,16 +169,18 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
         if (pflags & ProcessEngine_CollectCommandLine)
                 cmdline = StringBuffer_create(64);
         for (int i = 0; i < treesize; i++) {
-                pt[i].pid          = pinfo[i].p_pid;
-                pt[i].ppid         = pinfo[i].p_ppid;
-                pt[i].cred.uid     = pinfo[i].p_ruid;
-                pt[i].cred.euid    = pinfo[i].p_uid;
-                pt[i].cred.gid     = pinfo[i].p_rgid;
-                pt[i].threads      = pinfo[i].p_nlwps;
-                pt[i].uptime       = systeminfo.time / 10. - pinfo[i].p_ustart_sec;
-                pt[i].cpu.time     = pinfo[i].p_rtime_sec * 10 + (double)pinfo[i].p_rtime_usec / 100000.;
-                pt[i].memory.usage = (uint64_t)pinfo[i].p_vm_rssize * (uint64_t)pagesize;
-                pt[i].zombie       = pinfo[i].p_stat == SZOMB ? true : false;
+                pt[i].pid              = pinfo[i].p_pid;
+                pt[i].ppid             = pinfo[i].p_ppid;
+                pt[i].cred.uid         = pinfo[i].p_ruid;
+                pt[i].cred.euid        = pinfo[i].p_uid;
+                pt[i].cred.gid         = pinfo[i].p_rgid;
+                pt[i].threads          = pinfo[i].p_nlwps;
+                pt[i].uptime           = systeminfo.time / 10. - pinfo[i].p_ustart_sec;
+                pt[i].cpu.time         = pinfo[i].p_rtime_sec * 10 + (double)pinfo[i].p_rtime_usec / 100000.;
+                pt[i].memory.usage     = (uint64_t)pinfo[i].p_vm_rssize * (uint64_t)pagesize;
+                pt[i].zombie           = pinfo[i].p_stat == SZOMB ? true : false;
+                pt[i].read.operations  = pinfo[i].p_uru_inblock;
+                pt[i].write.operations = pinfo[i].p_uru_oublock;
                 if (pflags & ProcessEngine_CollectCommandLine) {
                         char **args = kvm_getargv2(kvm_handle, &pinfo[i], 0);
                         if (args) {
@@ -188,7 +190,7 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
                                 if (StringBuffer_length(cmdline))
                                         pt[i].cmdline = Str_dup(StringBuffer_toString(StringBuffer_trim(cmdline)));
                         }
-                        if (! pt[i].cmdline || ! *pt[i].cmdline) {
+                        if (STR_UNDEF(pt[i].cmdline)) {
                                 FREE(pt[i].cmdline);
                                 pt[i].cmdline = Str_dup(pinfo[i].p_comm);
                         }
@@ -227,12 +229,12 @@ boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
         size_t len = sizeof(struct uvmexp_sysctl);
         if (sysctl(mib, 2, &vm, &len, NULL, 0) == -1) {
                 LogError("system statistic error -- cannot get memory usage: %s\n", STRERROR);
-                si->swap_max = 0ULL;
+                si->swap.size = 0ULL;
                 return false;
         }
-        si->total_mem = (uint64_t)(vm.active + vm.wired) * (uint64_t)vm.pagesize;
-        si->swap_max = (uint64_t)vm.swpages * (uint64_t)vm.pagesize;
-        si->total_swap = (uint64_t)vm.swpginuse * (uint64_t)vm.pagesize;
+        si->memory.usage.bytes = (uint64_t)(vm.active + vm.wired) * (uint64_t)vm.pagesize;
+        si->swap.size = (uint64_t)vm.swpages * (uint64_t)vm.pagesize;
+        si->swap.usage.bytes = (uint64_t)vm.swpginuse * (uint64_t)vm.pagesize;
         return true;
 }
 
@@ -259,9 +261,9 @@ boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
         total     = total_new - total_old;
         total_old = total_new;
 
-        si->total_cpu_user_percent = (total > 0) ? (100. * (double)(cp_time[CP_USER] - cpu_user_old) / total) : -1.;
-        si->total_cpu_syst_percent = (total > 0) ? (100. * (double)(cp_time[CP_SYS] - cpu_syst_old) / total) : -1.;
-        si->total_cpu_wait_percent = 0; /* there is no wait statistic available */
+        si->cpu.usage.user = (total > 0) ? (100. * (double)(cp_time[CP_USER] - cpu_user_old) / total) : -1.;
+        si->cpu.usage.system = (total > 0) ? (100. * (double)(cp_time[CP_SYS] - cpu_syst_old) / total) : -1.;
+        si->cpu.usage.wait = 0; /* there is no wait statistic available */
 
         cpu_user_old = cp_time[CP_USER];
         cpu_syst_old = cp_time[CP_SYS];
