@@ -81,8 +81,7 @@
  *        Allows to skip the content match test for the content which was checked
  *        already to suppress duplicate events.
  *
- *    5.) size, checksum, timestamp, permissions, link speed, filesystem flags
- *        for the change observation test
+ *    5.) size, checksum, timestamp, permissions, link speed for the change observation test
  *
  * Data is stored in binary form in the statefile using the following format:
  *    <MAGIC><VERSION>{<SERVICE_STATE>}+
@@ -106,46 +105,92 @@ typedef enum {
         StateVersion0 = 0,
         StateVersion1,
         StateVersion2,
-        StateVersion3
+        StateVersion3,
+        StateVersion4
 } State_Version;
 
 
+/* Extended format version 4 */
+typedef struct mystate4 {
+        char               name[STRLEN];
+        int32_t            type;
+        int32_t            monitor;
+        int32_t            nstart;
+        int32_t            ncycle;
+        union {
+                struct {
+                        uint64_t atime;
+                        uint64_t ctime;
+                        uint64_t mtime;
+                        int32_t mode;
+                } directory;
+
+                struct {
+                        uint64_t inode;
+                        uint64_t readpos;
+                        uint64_t size;
+                        uint64_t atime;
+                        uint64_t ctime;
+                        uint64_t mtime;
+                        int32_t mode;
+                        MD_T hash;
+                } file;
+
+                struct {
+                        uint64_t atime;
+                        uint64_t ctime;
+                        uint64_t mtime;
+                        int32_t mode;
+                } fifo;
+
+                struct {
+                        int32_t mode;
+                } filesystem;
+
+                struct {
+                        int32_t duplex;
+                        int64_t speed;
+                        //FIXME: when Link API is moved from libmonit to monit, save also link bytes in/out and packets in/out history, so the network statistics is not reset on each monit reload
+                } net;
+        } priv;
+} State4_T;
+
+
 /* Extended format version 3 */
 typedef struct mystate3 {
         char               name[STRLEN];
-        int                type;
-        int                monitor;
-        int                nstart;
-        int                ncycle;
+        int32_t            type;
+        int32_t            monitor;
+        int32_t            nstart;
+        int32_t            ncycle;
         union {
                 struct {
-                        time_t timestamp;
-                        int mode;
+                        uint64_t timestamp;
+                        int32_t mode;
                 } directory;
 
                 struct {
-                        unsigned long long inode;
-                        unsigned long long readpos;
-                        unsigned long long size;
-                        unsigned long long timestamp;
-                        int mode;
+                        uint64_t inode;
+                        uint64_t readpos;
+                        uint64_t size;
+                        uint64_t timestamp;
+                        int32_t mode;
                         MD_T hash;
                 } file;
 
                 struct {
-                        unsigned long long timestamp;
-                        int mode;
+                        uint64_t timestamp;
+                        int32_t mode;
                 } fifo;
 
                 struct {
-                        int mode;
-                        int flags;
+                        int32_t mode;
+                        int32_t flags; // Obsolete since Monit 5.21.0
                 } filesystem;
 
                 struct {
-                        int duplex;
-                        long long speed;
-                        //FIXME: when Link API is moved from libmonit to monit, save also link bytes in/out and packets in/out history, so the network statistics is not reset on each monit reload
+                        int32_t duplex;
+                        int64_t speed;
                 } net;
         } priv;
 } State3_T;
@@ -158,14 +203,14 @@ typedef struct mystate3 State2_T;
 /* Extended format version 1 */
 typedef struct mystate1 {
         char               name[STRLEN];
-        int                type;
-        int                monitor;
-        int                nstart;
-        int                ncycle;
+        int32_t            type;
+        int32_t            monitor;
+        int32_t            nstart;
+        int32_t            ncycle;
         union {
                 struct {
-                        unsigned long long inode;
-                        unsigned long long readpos;
+                        uint64_t inode;
+                        uint64_t readpos;
                 } file;
         } priv;
 } State1_T;
@@ -173,12 +218,12 @@ typedef struct mystate1 {
 
 /* Format version 0 (Monit <= 5.3) */
 typedef struct mystate0 {
-        char               name[STRLEN];
-        int                mode;                // obsolete since Monit 5.1
-        int                nstart;
-        int                ncycle;
-        int                monitor;
-        unsigned long long error;               // obsolete since Monit 5.0
+        char     name[STRLEN];
+        int32_t  mode;                // obsolete since Monit 5.1
+        int32_t  nstart;
+        int32_t  ncycle;
+        int32_t  monitor;
+        uint64_t error;               // obsolete since Monit 5.0
 } State0_T;
 
 
@@ -212,16 +257,29 @@ static void _updateMonitor(Service_T S, Monitor_State monitor) {
 }
 
 
-static void _updateFilePosition(Service_T S, unsigned long long inode, unsigned long long readpos) {
-        S->inf->priv.file.inode = (ino_t)inode;
-        S->inf->priv.file.readpos = (off_t)readpos;
+static void _updateFilePosition(Service_T S, uint64_t inode, uint64_t readpos) {
+        S->inf.file->inode = (ino_t)inode;
+        S->inf.file->readpos = (off_t)readpos;
 }
 
 
-static void _updateTimestamp(Service_T S, unsigned long long timestamp) {
+static void _updateTimestamp(Service_T S, uint64_t atime, uint64_t ctime, uint64_t mtime) {
         for (Timestamp_T t = S->timestamplist; t; t = t->next) {
                 if (t->test_changes) {
-                        t->timestamp = (time_t)timestamp;
+                        switch (t->type) {
+                                case Timestamp_Access:
+                                        t->lastTimestamp = (time_t)atime;
+                                        break;
+                                case Timestamp_Change:
+                                        t->lastTimestamp = (time_t)ctime;
+                                        break;
+                                case Timestamp_Modification:
+                                        t->lastTimestamp = (time_t)mtime;
+                                        break;
+                                default:
+                                        t->lastTimestamp = (time_t)MAX(ctime, mtime);
+                                        break;
+                        }
                         t->initialized = true;
                 }
         }
@@ -234,7 +292,7 @@ static void _updatePermission(Service_T S, int mode) {
 }
 
 
-static void _updateSize(Service_T S, unsigned long long size) {
+static void _updateSize(Service_T S, uint64_t size) {
         for (Size_T s = S->sizelist; s; s = s->next) {
                 if (s->test_changes) {
                         s->size = size;
@@ -247,18 +305,12 @@ static void _updateSize(Service_T S, unsigned long long size) {
 static void _updateChecksum(Service_T S, char *hash) {
         if (S->checksum && S->checksum->test_changes) {
                 S->checksum->initialized = false;
-                strncpy(S->checksum->hash, hash, sizeof(S->checksum->hash));
+                strncpy(S->checksum->hash, hash, sizeof(S->checksum->hash) - 1);
         }
 }
 
 
-static void _updateFilesystemFlags(Service_T S, int flags) {
-        if (S->fsflaglist)
-                S->inf->priv.filesystem.flags = flags;
-}
-
-
-static void _updateLinkSpeed(Service_T S, int duplex, long long speed) {
+static void _updateLinkSpeed(Service_T S, int32_t duplex, int64_t speed) {
         for (LinkSpeed_T l = S->linkspeedlist; l; l = l->next) {
                 l->duplex = duplex;
                 l->speed = speed;
@@ -266,10 +318,58 @@ static void _updateLinkSpeed(Service_T S, int duplex, long long speed) {
 }
 
 
+static void _restoreV4() {
+        // System header
+        if (read(file, &booted, sizeof(booted)) != sizeof(booted)) {
+                THROW(IOException, "Unable to read system boot time");
+        }
+        // Services state
+        State4_T state;
+        while (read(file, &state, sizeof(state)) == sizeof(state)) {
+                Service_T service = Util_getService(state.name);
+                if (service && service->type == state.type) {
+                        _updateStart(service, state.nstart, state.ncycle);
+                        _updateMonitor(service, state.monitor);
+                        switch (service->type) {
+                                case Service_Directory:
+                                        _updatePermission(service, state.priv.directory.mode);
+                                        _updateTimestamp(service, state.priv.directory.atime, state.priv.directory.ctime, state.priv.directory.mtime);
+                                        break;
+
+                                case Service_Fifo:
+                                        _updatePermission(service, state.priv.fifo.mode);
+                                        _updateTimestamp(service, state.priv.fifo.atime, state.priv.fifo.ctime, state.priv.fifo.mtime);
+                                        break;
+
+                                case Service_File:
+                                        _updatePermission(service, state.priv.file.mode);
+                                        _updateTimestamp(service, state.priv.file.atime, state.priv.file.ctime, state.priv.file.mtime);
+                                        _updateFilePosition(service, state.priv.file.inode, state.priv.file.readpos);
+                                        _updateSize(service, state.priv.file.size);
+                                        _updateChecksum(service, state.priv.file.hash);
+                                        break;
+
+                                case Service_Filesystem:
+                                        _updatePermission(service, state.priv.filesystem.mode);
+                                        break;
+
+                                case Service_Net:
+                                        _updateLinkSpeed(service, state.priv.net.duplex, state.priv.net.speed);
+                                        break;
+
+                                default:
+                                        break;
+                        }
+                }
+        }
+}
+
+
 static void _restoreV3() {
         // System header
-        if (read(file, &booted, sizeof(booted)) != sizeof(booted))
+        if (read(file, &booted, sizeof(booted)) != sizeof(booted)) {
                 THROW(IOException, "Unable to read system boot time");
+        }
         // Services state
         State3_T state;
         while (read(file, &state, sizeof(state)) == sizeof(state)) {
@@ -280,17 +380,14 @@ static void _restoreV3() {
                         switch (service->type) {
                                 case Service_Directory:
                                         _updatePermission(service, state.priv.directory.mode);
-                                        _updateTimestamp(service, state.priv.directory.timestamp);
                                         break;
 
                                 case Service_Fifo:
                                         _updatePermission(service, state.priv.fifo.mode);
-                                        _updateTimestamp(service, state.priv.fifo.timestamp);
                                         break;
 
                                 case Service_File:
                                         _updatePermission(service, state.priv.file.mode);
-                                        _updateTimestamp(service, state.priv.file.timestamp);
                                         _updateFilePosition(service, state.priv.file.inode, state.priv.file.readpos);
                                         _updateSize(service, state.priv.file.size);
                                         _updateChecksum(service, state.priv.file.hash);
@@ -298,7 +395,6 @@ static void _restoreV3() {
 
                                 case Service_Filesystem:
                                         _updatePermission(service, state.priv.filesystem.mode);
-                                        _updateFilesystemFlags(service, state.priv.filesystem.flags);
                                         break;
 
                                 case Service_Net:
@@ -326,17 +422,14 @@ static void _restoreV2() {
                         switch (service->type) {
                                 case Service_Directory:
                                         _updatePermission(service, state.priv.directory.mode);
-                                        _updateTimestamp(service, state.priv.directory.timestamp);
                                         break;
 
                                 case Service_Fifo:
                                         _updatePermission(service, state.priv.fifo.mode);
-                                        _updateTimestamp(service, state.priv.fifo.timestamp);
                                         break;
 
                                 case Service_File:
                                         _updatePermission(service, state.priv.file.mode);
-                                        _updateTimestamp(service, state.priv.file.timestamp);
                                         _updateFilePosition(service, state.priv.file.inode, state.priv.file.readpos);
                                         _updateSize(service, state.priv.file.size);
                                         _updateChecksum(service, state.priv.file.hash);
@@ -344,7 +437,6 @@ static void _restoreV2() {
 
                                 case Service_Filesystem:
                                         _updatePermission(service, state.priv.filesystem.mode);
-                                        _updateFilesystemFlags(service, state.priv.filesystem.flags);
                                         break;
 
                                 case Service_Net:
@@ -420,21 +512,26 @@ void State_close() {
 void State_save() {
         TRY
         {
-                if (ftruncate(file, 0L) == -1)
+                if (ftruncate(file, 0L) == -1) {
                         THROW(IOException, "Unable to truncate");
-                if (lseek(file, 0L, SEEK_SET) == -1)
+                }
+                if (lseek(file, 0L, SEEK_SET) == -1) {
                         THROW(IOException, "Unable to seek");
-                int magic = 0;
-                if (write(file, &magic, sizeof(magic)) != sizeof(magic))
+                }
+                int32_t magic = 0;
+                if (write(file, &magic, sizeof(magic)) != sizeof(magic)) {
                         THROW(IOException, "Unable to write magic");
+                }
                 // Save always using the latest format version
-                int version = StateVersion3;
-                if (write(file, &version, sizeof(version)) != sizeof(version))
+                int32_t version = StateVersion4;
+                if (write(file, &version, sizeof(version)) != sizeof(version)) {
                         THROW(IOException, "Unable to write format version");
-                if (write(file, &systeminfo.booted, sizeof(systeminfo.booted)) != sizeof(systeminfo.booted))
+                }
+                if (write(file, &systeminfo.booted, sizeof(systeminfo.booted)) != sizeof(systeminfo.booted)) {
                         THROW(IOException, "Unable to write system boot time");
+                }
                 for (Service_T service = servicelist; service; service = service->next) {
-                        State3_T state;
+                        State4_T state;
                         memset(&state, 0, sizeof(state));
                         snprintf(state.name, sizeof(state.name), "%s", service->name);
                         state.type = service->type;
@@ -443,32 +540,42 @@ void State_save() {
                         state.ncycle = service->ncycle;
                         switch (service->type) {
                                 case Service_Directory:
-                                        state.priv.directory.timestamp = (unsigned long long)service->inf->priv.directory.timestamp;
-                                        if (service->perm)
+                                        state.priv.directory.atime = (uint64_t)service->inf.directory->timestamp.access;
+                                        state.priv.directory.ctime = (uint64_t)service->inf.directory->timestamp.change;
+                                        state.priv.directory.mtime = (uint64_t)service->inf.directory->timestamp.modify;
+                                        if (service->perm) {
                                                 state.priv.directory.mode = service->perm->perm;
+                                        }
                                         break;
 
                                 case Service_Fifo:
-                                        state.priv.fifo.timestamp = (unsigned long long)service->inf->priv.fifo.timestamp;
-                                        if (service->perm)
+                                        state.priv.fifo.atime = (uint64_t)service->inf.fifo->timestamp.access;
+                                        state.priv.fifo.ctime = (uint64_t)service->inf.fifo->timestamp.change;
+                                        state.priv.fifo.mtime = (uint64_t)service->inf.fifo->timestamp.modify;
+                                        if (service->perm) {
                                                 state.priv.fifo.mode = service->perm->perm;
+                                        }
                                         break;
 
                                 case Service_File:
-                                        state.priv.file.inode = service->inf->priv.file.inode;
-                                        state.priv.file.readpos = service->inf->priv.file.readpos;
-                                        state.priv.file.size = (unsigned long long)service->inf->priv.file.size;
-                                        state.priv.file.timestamp = (unsigned long long)service->inf->priv.file.timestamp;
-                                        if (service->checksum)
-                                                strncpy(state.priv.file.hash, service->inf->priv.file.cs_sum, sizeof(state.priv.file.hash));
-                                        if (service->perm)
+                                        state.priv.file.inode = service->inf.file->inode;
+                                        state.priv.file.readpos = service->inf.file->readpos;
+                                        state.priv.file.size = (uint64_t)service->inf.file->size;
+                                        state.priv.file.atime = (uint64_t)service->inf.file->timestamp.access;
+                                        state.priv.file.ctime = (uint64_t)service->inf.file->timestamp.change;
+                                        state.priv.file.mtime = (uint64_t)service->inf.file->timestamp.modify;
+                                        if (service->checksum) {
+                                                strncpy(state.priv.file.hash, service->inf.file->cs_sum, sizeof(state.priv.file.hash) - 1);
+                                        }
+                                        if (service->perm) {
                                                 state.priv.file.mode = service->perm->perm;
+                                        }
                                         break;
 
                                 case Service_Filesystem:
-                                        if (service->perm)
+                                        if (service->perm) {
                                                 state.priv.filesystem.mode = service->perm->perm;
-                                        state.priv.filesystem.flags = service->inf->priv.filesystem.flags;
+                                        }
                                         break;
 
                                 case Service_Net:
@@ -481,11 +588,13 @@ void State_save() {
                                 default:
                                         break;
                         }
-                        if (write(file, &state, sizeof(state)) != sizeof(state))
+                        if (write(file, &state, sizeof(state)) != sizeof(state)) {
                                 THROW(IOException, "Unable to write service state");
+                        }
                 }
-                if (fsync(file))
+                if (fsync(file)) {
                         THROW(IOException, "Unable to sync -- %s", STRERROR);
+                }
         }
         ELSE
         {
@@ -497,23 +606,27 @@ void State_save() {
 
 void State_restore() {
         /* Ignore empty state file */
-        if ((lseek(file, 0L, SEEK_END) == 0))
+        if ((lseek(file, 0L, SEEK_END) == 0)) {
                 return;
+        }
         TRY
         {
-                if (lseek(file, 0L, SEEK_SET) == -1)
+                if (lseek(file, 0L, SEEK_SET) == -1) {
                         THROW(IOException, "Unable to seek");
-                int magic;
-                if (read(file, &magic, sizeof(magic)) != sizeof(magic))
+                }
+                int32_t magic;
+                if (read(file, &magic, sizeof(magic)) != sizeof(magic)) {
                         THROW(IOException, "Unable to read magic");
+                }
                 if (magic > 0) {
                         // The statefile format of Monit <= 5.3, the magic is number of services, followed by State0_T structures
                         _restoreV0(magic);
                 } else {
                         // The extended statefile format (Monit >= 5.4)
-                        int version;
-                        if (read(file, &version, sizeof(version)) != sizeof(version))
+                        int32_t version;
+                        if (read(file, &version, sizeof(version)) != sizeof(version)) {
                                 THROW(IOException, "Unable to read version");
+                        }
                         switch (version) {
                                 case StateVersion1:
                                         _restoreV1();
@@ -524,6 +637,9 @@ void State_restore() {
                                 case StateVersion3:
                                         _restoreV3();
                                         break;
+                                case StateVersion4:
+                                        _restoreV4();
+                                        break;
                                 default:
                                         LogWarning("State file '%s': incompatible version %d\n", Run.files.state, version);
                                         break;
